@@ -574,6 +574,26 @@ static gint do_reselect(gpointer data){
 	reselecttimer=0;
 	return FALSE;
 }
+static gint delayed_selection(gpointer data){
+	GtkTreeView * treeview=(GtkTreeView * )data;
+        GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+#ifdef DEBUG
+	 printf("delayed selection...\n");
+#endif
+     	gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+#if DEBUG_IN_DND
+	/* DEBUG must be enabled in dnd.c for this to work: */
+#ifdef DEBUG
+	{
+	    extern void the_sl(void);
+	    the_sl();
+	}
+#endif
+#endif
+	
+	return FALSE;
+}
+
 
 	
 gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer data)
@@ -813,10 +833,21 @@ done:
 	    /* this enables dnd(right), but also selection(wrong)*/
 #endif
 	case 1:
-	  if (!(event->state&GDK_CONTROL_MASK) && !(event->state&GDK_SHIFT_MASK)) {
-	    clear_dnd_selection_list();
-    	    gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
-	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &treepath, NULL, NULL, NULL))	
+    	    clear_dnd_selection_list();
+	    if (tree_details->preferences & ENABLE_GTK_CLICK){
+		 g_timeout_add_full(0, 50, (GtkFunction) delayed_selection, (gpointer) treeview, NULL);
+	    } else { /* !ENABLE_GTK_CLICK */
+		gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+	        if (event->state & GDK_CONTROL_MASK || event->state & GDK_SHIFT_MASK){
+		    if (event->state & GDK_SHIFT_MASK){
+		       g_timeout_add_full(0, 50, (GtkFunction) delayed_selection, (gpointer) treeview, NULL);
+		    }
+		} else {
+		    reselecttimer = g_timeout_add_full(0, 150, (GtkFunction) do_reselect, (gpointer) treeview, NULL);
+		    /*printf("DBG:reselecttimer = %d\n",reselecttimer );*/
+		}
+	    }
+	    if(gtk_tree_view_get_path_at_pos(treeview, event->x,  event->y, &treepath, NULL, NULL, NULL));	
 	    if (!is_in_dnd_selection(treepath)){
 		if (gtk_tree_model_get_iter (treemodel,&iter,treepath)) 
 		{
@@ -829,14 +860,15 @@ done:
 		if (clickpath) gtk_tree_path_free(clickpath);
 		clickpath=NULL;
 	    }
-	    if (!(tree_details->preferences & ENABLE_GTK_CLICK)) 
-	    {
-		    reselecttimer = g_timeout_add_full(0, 150, (GtkFunction) do_reselect, (gpointer) treeview, NULL);
+#if DEBUG_IN_DND
+	/* DEBUG must be enabled in dnd.c for this to work: */
 #ifdef DEBUG
-		    printf("DBG:reselecttimer = %d\n",reselecttimer );
-#endif
+	    {
+	      extern void the_sl(void);
+	      the_sl();
 	    }
-	  }
+#endif
+#endif
 	    return FALSE;
     }
     return FALSE;
